# Simulation of SFS in a haploid model with selection
# Population size, selection and mutation can be variable

# Sylvain Glémin: sylvain.glemin@univ-montp2.fr


##################### MUTATION FUNCTION  #########################################

# We assume an infinite site mode
# So mutation enters in the population only on one copy and only if the population is monomorphic

mute <- function(u,N){
  nbmut <- rpois(1,N*u)
  if(nbmut>0) return(1/N) else return(0)
}


##################### SELECTION FUNCTION   #########################################

select <- function(p,s){   
  return( p*(1+s)/(1+s*p) )
}


##################### SIMULATION FUNCTION   #########################################

# Arguments
# N_simul: number of simulations
# N_chrom: number of sampled chromosomes
# T_simul: duration of the simulation
# N: vector of population size. Must be of the size of T_simul
# sel: vector of selection coefficient. Must be of the size of T_simul
# mu: vector of mutation rate. Must be of the size of T_simul

# Value
# Return a list with three arguments
# list$sfs: vector of of SFS
# list$fixed: number of fixed mutation in the sample
# list$nbgenmut: number of generations during which mutation was allowed (for normalization)

# To compute the subsitution rate
# Rate = fixed/nbgenmut
# To obtain the number of substitution
# Nb_subst = fixed x nbgenmut / (Tsim x Nsim)


onesimul <- function(N_chrom=10,T_simul=500,N=rep(100,T_simul),sel=rep(0.01,T_simul),mu=rep(0.001,T_simul)){
  tabsimul <- vector("numeric")
  # Initialisation
  # Simulation
  
  sim1 <- function(j){
    p <- rpois(1,N[j]*mu[j]) / N[j]
    if(p>0){
      for (i in j:(T_simul-1)) {
        p <- select(p,sel[i])
        p <- rbinom(1,N[i+1],p)/N[i+1]
        if(p==0 || p==1) break
      }
    }
    # Sampling
    return(rbinom(1,N_chrom,p))
  }
  tabsimul <- mapply(sim1,c(1:(T_simul-1)))
  count <- hist(tabsimul,seq(-0.5,N_chrom+0.5,1),plot=F)$counts
  sfs <- count[2:(N_chrom+1)]
  return(sfs)
}



simulSFS <- function(N_simul=100,N_chrom=10,T_simul=500,N=rep(100,T_simul),sel=rep(0.01,T_simul),mu=rep(0.001,T_simul)){
  sim <- mapply(onesimul,rep(N_chrom,N_simul),MoreArgs = list(T_simul=T_simul,N=N,sel=sel,mu=mu))
  sfs <- rowSums(sim)
  return(sfs)
}



##################### EXAMPLES   #########################################



# Constant population size and selection
for(i in 1:100){
  simN <- simulSFS(N_simul = 5000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(0,1000) , mu = rep(0.0005,1000))
  simWS <- simulSFS(N_simul = 5000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(0.005,1000) , mu = rep(0.001,1000))
  simSW <- simulSFS(N_simul = 5000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(-0.005,1000) , mu = rep(0.002,1000))
  sfsN <- simN
  sfsWS <- simWS
  sfsSW <- simSW
  write.table(x = matrix(c(sfsN,sfsWS,sfsSW,rep(0.5,20)),20,4),file = paste("sim_constantB=0.5_",i,".txt",sep=""),quote = F,row.names = F,col.names = c("Sn","Sws","Ssw","GC"),sep="\t")
  print(i)
}

# Increase in selection
for(i in 1:100){
  simN <- simulSFS(N_simul = 5000,N_chrom = 20,T_simul = 600 ,N = rep(50,600), sel = rep(0,600) ,mu = rep(0.0005,600))
  simWS <- simulSFS(N_simul = 5000,N_chrom = 20,T_simul = 600 ,N = rep(50,600), sel = c(rep(0.005,200),rep(0.01,400)) ,mu = rep(0.001,600))
  simSW <- simulSFS(N_simul = 5000,N_chrom = 20,T_simul = 600 ,N = rep(50,600), sel = c(rep(-0.005,200),rep(-0.01,400)) ,mu = rep(0.002,600))
  sfsN <- simN
  sfsWS <- simWS
  sfsSW <- simSW
  write.table(x = matrix(c(sfsN,sfsWS,sfsSW,rep(0.5,20)),20,4),file = paste("sim_T0=200_T1=400_B0=0.5_B1=1_",i,".txt",sep=""),quote = F,row.names = F,col.names = c("Sn","Sws","Ssw","GC"),sep="\t")
  print(i)
}

# Demographic change: bottleneck + expansion
for(i in 1:100){
  simN <- simulSFS(N_simul = 5000,N_chrom = 20,T_simul = 1000 ,N = c(rep(100,700),rep(25,100),rep(200,200)), sel = rep(0,1000) ,mu = rep(0.0005,1000))
  simWS <- simulSFS(N_simul = 5000,N_chrom = 20,T_simul = 1000 ,N = c(rep(100,700),rep(25,100),rep(200,200)), sel = rep(0.005,1000)  ,mu = rep(0.001,1000))
  simSW <- simulSFS(N_simul = 5000,N_chrom = 20,T_simul = 1000 ,N = c(rep(100,700),rep(25,100),rep(200,200)), sel = rep(-0.005,1000)  ,mu = rep(0.002,1000))
  sfsN <- simN
  sfsWS <- simWS
  sfsSW <- simSW
  write.table(x = matrix(c(sfsN,sfsWS,sfsSW,rep(0.5,20)),20,4),file = paste("sim_bottlenecks_expansion",i,".txt",sep=""),quote = F,row.names = F,col.names = c("Sn","Sws","Ssw","GC"),sep="\t")
  print(i)
}

# Change in mutation bias
for(i in 1:100){
  simN <- simulSFS(N_simul = 5000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(0,1000) , mu = rep(0.0005,1000))
  simWS <- simulSFS(N_simul = 5000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(0.005,1000) , mu = c(rep(0.001,500),rep(0.002,500)))
  simSW <- simulSFS(N_simul = 5000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(-0.005,1000) , mu = rep(0.002,1000))
  sfsN <- simN
  sfsWS <- simWS
  sfsSW <- simSW
  write.table(x = matrix(c(sfsN,sfsWS,sfsSW,rep(0.5,20)),20,4),file = paste("sim_constantB=0.5_T0=500_T1=500,mu0=2_mu1=1_",i,".txt",sep=""),quote = F,row.names = F,col.names = c("Sn","Sws","Ssw","GC"),sep="\t")
  print(i)
}



# Joint simulation of gBGC and SCU
for(i in 1:100){
  simNN <- simulSFS(N_simul = 2000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(0,1000) , mu = rep(0.0005,1000))
  simNUP <- simulSFS(N_simul = 2000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(0.005,1000) , mu = rep(0.0005,1000))
  simNPU <- simulSFS(N_simul = 2000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(-0.005,1000) , mu = rep(0.0005,1000))
  simWSN <- simulSFS(N_simul = 2000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(0.005,1000) , mu = rep(0.001,1000))
  simWSUP <- simulSFS(N_simul = 2000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(0.01,1000) , mu = rep(0.001,1000))
  simWSPU <- simulSFS(N_simul = 2000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(0,1000) , mu = rep(0.001,1000))
  simSWN <- simulSFS(N_simul = 2000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(-0.005,1000) , mu = rep(0.002,1000))
  simSWUP <- simulSFS(N_simul = 2000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(0,1000) , mu = rep(0.002,1000))
  simSWPU <- simulSFS(N_simul = 2000,N_chrom = 20,T_simul = 1000 ,N = rep(50,1000), sel = rep(-0.01,1000) , mu = rep(0.002,1000))
  write.table(x = matrix(c(c(simNN,simNUP,simNPU,simWSN,simWSUP,simWSPU,simSWN,simSWUP,simSWPU),rep(0.5,20)),20,10),file = paste("sim_gBGCSCU_A_",i,".txt",sep=""),quote = F,row.names = F,col.names = c("Snn","Snup","Snpu","Swsn","Swsup","Swspu","Sswn","Sswup","Sswpu","GC"),sep="\t")
  print(i)
}